--[[
 * Copyright (C) 2016 Ricky K. Thomson
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * u should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 --]]


require("textures")


function love.load(args)

	starfield = require("starfield")

	debug = false
	showtext = true
	
	love.mouse.setVisible(false)
	love.mouse.setGrabbed(true)

	game = {}

	game.width, game.height, game.flags = love.window.getMode( )
	game.seed = nil
	game.max_fps = game.flags.refreshrate
	game.min_dt = 1/game.max_fps
	game.next_time = love.timer.getTime()

	starfield:populate()
	
end


function love.update(dt)
	--cap fps
	game.next_time = game.next_time + game.min_dt


	if love.keyboard.isDown("[") then
		starfield:speedAdjust(-4, dt)	
	elseif love.keyboard.isDown("]") then			
		starfield:speedAdjust(4, dt)
	end

	--update starfield
	starfield:update(dt)

end



	
function love.draw()

	--draw starfield
	starfield:draw(0,0)

	if showtext then
		love.graphics.setColor(255,255,255,200)
		love.graphics.print("scrolling starfield\n\npress [ to decrease speed\npress ] to increase speed\n\n press j for a new seed\npress space to toggle text display",10,10)
	end

	-- caps fps
	local cur_time = love.timer.getTime()
	if game.next_time <= cur_time then
		game.next_time = cur_time
		return
	end
	love.timer.sleep(game.next_time - cur_time)

end


function love.keypressed(key)
	
	if key == "j" then
		starfield:setSeed()
		starfield:setColor()
		starfield:populate()
	end
	
	if key == "escape" then
		love.event.quit()
	end
	
	if key == "space" then
		showtext = not showtext
	end

end
